import unittest

from docker_tag_naming.utils import get_latest_version


class BasicTest(unittest.TestCase):
    def test_has_version(self):
        version = get_latest_version('andresriancho/w3af', 'develop')
        self.assertIsInstance(version.version_number, int)
        self.assertIsInstance(version.commit, basestring)
        self.assertEqual(version.branch, 'develop')

    def test_no_version(self):
        version = get_latest_version('andresriancho/django-moth', 'develop')
        self.assertIsNone(version)

    def test_invalid_branch(self):
        version = get_latest_version('andresriancho/django-moth', 'foobar')
        self.assertIsNone(version)



import requests
import logging
import os

from requests.exceptions import RequestException
from .constants import VERSION_FORMAT

API_URL = 'https://registry.hub.docker.com/v1/repositories'


class Version(object):
    def __init__(self, version_number, commit, branch):
        self.version_number = version_number
        self.commit = commit
        self.branch = branch

    def __str__(self):
        return VERSION_FORMAT % (self.version_number, self.commit, self.branch)

    def bump(self):
        self.version_number += 1


def version_parser(version_string):
    """
    :param version_string: The version tag as returned by the registry/hub API
    :return: A tuple with the parsed version / an exception when the version
             format is unknown/incorrect
    """
    version, commit, branch = version_string.split('-', 3)

    # Remove the "v"
    version_number = version[1:]
    version_number = int(version_number)

    return Version(version_number, commit, branch)


def get_all_tags_with_auth(image_name, branch=None):
    """
    Get the tag information using authentication credentials provided by the
    user.

    :param image_name: The image name to query
    :param branch: The branch to filter by
    :return: A list of Version instances, latest first
    """
    logging.debug('Getting %s with authentication' % image_name)
    url = '%s/%s/images' % (API_URL, image_name)

    registry_user = os.environ.get('REGISTRY_USER')
    registry_pass = os.environ.get('REGISTRY_PASS')

    if registry_user is None or registry_pass is None:
        msg = ('The docker image seems to be private and no authentication'
               ' credentials were provided. Please set REGISTRY_USER and'
               ' REGISTRY_PASS environment variables and try again.')
        raise DockerTagNamingException(msg)

    user_credentials = (registry_user, registry_pass)

    headers = {'X-Docker-Token': 'true',
               'Accept': 'application/json',
               'Content-Type': 'application/json'}

    try:
        response = requests.get(url, auth=user_credentials, headers=headers)
    except RequestException, re:
        raise DockerTagNamingException('HTTP request exception "%s"' % re)

    if response.status_code in (404, 401):
        msg = ('Received unexpected status code %s from the registry'
               ' REST API after sending authentication credentials, the image'
               ' might not exist or is private.')
        raise AuthException(msg % response.status_code)

    token = response.headers.get('x-docker-token')
    endpoint = response.headers.get('x-docker-endpoints')

    if token is None:
        raise DockerTagNamingException('No token returned by the REST API')

    if endpoint is None:
        raise DockerTagNamingException('No docker endpoint returned by the'
                                       ' REST API')

    url = 'https://{0}/v1/repositories/{1}/tags'.format(endpoint, image_name)
    headers['Authorization'] = 'Token %s' % token
    try:
        response = requests.get(url, headers=headers)
    except RequestException, re:
        raise DockerTagNamingException('HTTP request exception "%s"' % re)

    if response.status_code in (404,):
        msg = ('Received unexpected status code %s from the registry'
               ' REST API, the image might not exist in the registry but was'
               ' registered in the hub. Try doing a "docker push %s"')
        raise AuthException(msg % (response.status_code, image_name))

    if response.status_code != 200:
        msg = ('Received unexpected status code %s from the registry'
               ' REST API. Please report a bug.')
        raise AuthException(msg % response.status_code)

    try:
        json_data = response.json()
    except ValueError:
        msg = 'JSON decode failed! Raw data is: "%s". Please report a bug.'
        raise DockerTagNamingException(msg % (response.content[:25]).strip())

    #
    #   Note that the registry returns a different format
    #
    output = []

    for tag_name in json_data.keys():
        try:
            version = version_parser(tag_name)
        except Exception, e:
            msg = 'Ignoring version tag "%s" with incorrect format: "%s"'
            logging.debug(msg % (tag_name, e))
            continue

        if branch is not None:
            if version.branch != branch:
                continue

        output.append(version)

    def sort_func(version_a, version_b):
        return cmp(version_b.version_number, version_a.version_number)

    output.sort(sort_func)

    return output


def get_all_tags_no_auth(image_name, branch=None):
    """
    Try to get the tags without any authentication, this does work only for
    public images.

    GET /v1/repositories/<namespace>/<repository_name>/tags

    :param image_name: The docker image name
    :param branch: The branch to filter by
    :return: A list of Version instances, latest first
    """
    output = []
    logging.debug('Getting %s without authentication' % image_name)

    spec = '/%s/tags' % image_name
    headers = {'Accept': 'application/json',
               'Content-Type': 'application/json'}

    try:
        # This works well for any public images:
        response = requests.get(API_URL + spec, headers=headers)
    except RequestException, re:
        raise DockerTagNamingException('HTTP request exception "%s"' % re)

    if response.status_code == 401:
        msg = ('Received unexpected status code %s from the registry'
               ' REST API, the image is private.')
        raise AuthException(msg % response.status_code)

    if response.status_code != 200:
        msg = ('Received unexpected status code %s from the registry'
               ' REST API, the image might not exist')
        raise DockerTagNamingException(msg % response.status_code)

    try:
        json_data = response.json()
    except ValueError:
        msg = 'JSON decode failed! Raw data is: "%s". Please report a bug.'
        raise DockerTagNamingException(msg % (response.content[:25]).strip())

    try:
        tag_names = [tag_info['name'] for tag_info in json_data]
    except Exception:
        msg = ('The JSON data does not contain the expected format!'
               ' Raw data is: "%s". Please report a bug.')
        raise DockerTagNamingException(msg % (response.content[:25]).strip())

    for tag_name in tag_names:

        try:
            version = version_parser(tag_name)
        except Exception, e:
            msg = 'Ignoring version tag "%s" with incorrect format: "%s"'
            logging.debug(msg % (tag_name, e))
            continue

        if branch is not None:
            if version.branch != branch:
                continue

        output.append(version)

    def sort_func(version_a, version_b):
        return cmp(version_b.version_number, version_a.version_number)

    output.sort(sort_func)

    return output


def get_all_tags(image_name, branch=None):
    """
    GET /v1/repositories/<namespace>/<repository_name>/tags

    :param image_name: The docker image name
    :param branch: The branch to filter by
    :return: A list of Version instances, latest first
    """
    try:
        return get_all_tags_no_auth(image_name, branch)
    except AuthException:
        return get_all_tags_with_auth(image_name, branch)


def get_latest_version(image_name, branch):
    all_tags = get_all_tags(image_name, branch=branch)
    if all_tags:
        return all_tags[0]
    return None


def version_bump(image_name, branch, commit):
    version = get_latest_version(image_name, branch)
    if version is None:
        return Version(1, commit, branch)

    version.bump()
    version.commit = commit
    return version


class DockerTagNamingException(Exception):
    pass


class AuthException(DockerTagNamingException):
    pass


from __future__ import print_function

import sys

from docker_tag_naming.utils import Version, DockerTagNamingException


def run_forge(args):
    """
    Handle the forge subcommand
    :param args: Parsed command line arguments
    :return: None, output is written to console
    """
    version = args.version
    commit_id = args.commit_id
    branch = args.branch

    try:
        print(Version(version, commit_id, branch))
    except DockerTagNamingException, de:
        print(de)
        sys.exit(1)




import argparse

from .bump import run_bump
from .latest import run_latest
from .forge import run_forge
from .refresh import run_refresh


def parse_args_and_run():
    """
    :return: The parsed arguments
    """
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers(help='Docker-tag-naming sub-commands',
                                       dest='subparser_name')

    #
    #   Forge
    #
    parser_forge = subparsers.add_parser('forge',
                                         help='Create a new version tag')
    parser_forge.add_argument('--version', type=int, default=1,
                              help='Version number')
    parser_forge.add_argument('--commit-id', required=True,
                              help='Git commit id')
    parser_forge.add_argument('--branch', required=True,
                              help='The branch name (ie. master)')

    #
    #   Latest
    #
    parser_latest = subparsers.add_parser('latest',
                                          help='Query the latest tag in the'
                                               ' registry')
    parser_latest.add_argument('image',
                               help='The image to query (ie. username/image)')
    parser_latest.add_argument('branch',
                               help='The branch name (ie. master)')

    #
    #   Bump
    #
    parser_bump = subparsers.add_parser('bump',
                                        help='Query the latest tag in the'
                                             ' registry and return a +1')
    parser_bump.add_argument('image',
                             help='The image to bump (ie. username/image)')
    parser_bump.add_argument('branch',
                             help='The branch name (ie. master)')
    parser_bump.add_argument('--commit-id', required=True,
                             help='Git commit id for the newly created tag')

    #
    #   Refresh
    #
    parser_latest = subparsers.add_parser('refresh',
                                          help='Loop until the latest tag in'
                                               ' the registry changes')
    parser_latest.add_argument('image',
                               help='The image to query (ie. username/image)')
    parser_latest.add_argument('branch',
                               help='The branch name (ie. master)')

    args = parser.parse_args()

    {'bump': run_bump,
     'latest': run_latest,
     'forge': run_forge,
     'refresh': run_refresh}.get(args.subparser_name)(args)


def main():
    """
    Main entry point for the docker-tag-naming command, parse command line
    arguments, call functions and output the data.

    :return: Exit code
    """
    parse_args_and_run()



from __future__ import print_function

import sys
import time

from docker_tag_naming.utils import get_latest_version, DockerTagNamingException


def run_refresh(args):
    """
    Handle the refresh subcommand
    :param args: Parsed command line arguments
    :return: None, output is written to console
    """
    image = args.image
    branch = args.branch

    try:
        initial_version = get_latest_version(image, branch)
    except DockerTagNamingException, de:
        print(de)
        sys.exit(1)

    if initial_version is None:
        print('No version found')
        sys.exit(1)

    print('Initial version is %s , waiting for new release...' % initial_version)

    for _ in xrange(120):
        time.sleep(1)

        try:
            version = get_latest_version(image, branch)
        except DockerTagNamingException, de:
            print(de)
            sys.exit(1)

        if version.version_number != initial_version.version_number:
            print('New version found: %s' % version)
            sys.exit(0)

    print('Timeout: version did not change.')
    sys.exit(0)




#!/usr/bin/env python
import os

from setuptools import setup, find_packages
from os.path import join, dirname
from docker_tag_naming import __VERSION__


if os.environ.get('CIRCLECI', None) is not None:
    # monkey-patch distutils upload
    #
    # http://bugs.python.org/issue21722
    # https://github.com/tagcubeio/tagcube-cli/issues/4
    try:
        from distutils.command import upload as old_upload_module
        from ci.upload import upload as fixed_upload
        old_upload_module.upload = fixed_upload
    except ImportError:
        # In some cases I install tagcube-cli in CircleCI, but not from the
        # repository where the ci module is present
        pass


setup(
    name='docker-tag-naming',

    version=__VERSION__,
    license='GNU General Public License v2 (GPLv2)',
    platforms='Linux',

    description='Name and query docker tags',
    long_description=file(join(dirname(__file__), 'README.rst')).read(),

    author='Andres Riancho',
    author_email='andres.riancho@gmail.com',
    url='https://github.com/andresriancho/docker-tag-naming/',

    packages=find_packages(exclude=('ci',)),
    include_package_data=True,
    install_requires=['requests[security]>=2.7.0'],

    entry_points = {
        'console_scripts': [
            'docker-tag-naming = docker_tag_naming.main:main',
        ],
    },

    # https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
        'Natural Language :: English',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: System :: Monitoring'
    ],

    # In order to run this command: python setup.py test
    test_suite="nose.collector",
    tests_require="nose",
)




__VERSION__ = '1.0.5'


#
#   Disable requests logging
#
import logging

logging.getLogger('requests').setLevel(logging.WARNING)
urllib3_logger = logging.getLogger('urllib3')
urllib3_logger.setLevel(logging.CRITICAL)